var EXPORTED_SYMBOLS = ["dslrAlerts"];

const {classes: Cc, interfaces: Ci, utils: Cu} = Components;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "logger", "chrome://dslrnotifier2/content/modules/logger.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "dslrStatus", "chrome://dslrnotifier2/content/modules/status.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "Prefs", "chrome://dslrnotifier2/content/modules/prefs.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "Privacy", "chrome://dslrnotifier2/content/modules/privacy.jsm");

XPCOMUtils.defineLazyGetter(this, "PrivateBrowsing", function() {
	if (Cc["@mozilla.org/privatebrowsing;1"]) {
		return Cc["@mozilla.org/privatebrowsing;1"].getService(Ci.nsIPrivateBrowsingService);
	}
	else 
		return null;
});

// This only exists in Gecko 17 and up, so define our own lazy getter to set it to null if it doesn't exist instead of throwing an exception
XPCOMUtils.defineLazyGetter(this, "PrivateBrowsingUtils", function() {
	XPCOMUtils.defineLazyModuleGetter(this, "PrivateBrowsingUtils", "resource://gre/modules/PrivateBrowsingUtils.jsm");
	try {
		return PrivateBrowsingUtils;
	}
	catch(ex) {
		return null
	}
});

// This only exists in Firefox 20 and up, so define our own lazy getter to set it to null if it doesn't exist instead of throwing an exception
XPCOMUtils.defineLazyGetter(this, "RecentWindow", function() {
	XPCOMUtils.defineLazyModuleGetter(this, "RecentWindow", "resource:///modules/RecentWindow.jsm");
	try {
		return RecentWindow;
	}
	catch(ex) {
		return null
	}
});

var dslrAlerts = {
	init: function() {
		alertSlide.init();
	},
	
	alertCheck: function() {
		var topChange = dslrStatus.newsFlag;
		var imChange = dslrStatus.imFlag;
		var logChange = dslrStatus.loginFlag;
		var imSlide = Prefs.imSlide;
		var topSlide = Prefs.topSlide;
		var topPop = Prefs.topPop;
		var imPop = Prefs.poppref;
		var topSound = Prefs.topSound;
		var imSound = Prefs.imSound;
		var alertSlideValue = 0;
		var alertPopValue = 0;
		
		if (topChange && topSlide && !logChange) {
			alertSlideValue = alertSlideValue + 2;
		}

		if (topChange && topPop) {
			alertPopValue = alertPopValue + 2;
		}
	
		if (topChange && topSound) {
			alertSound(2);
		}
	
		if (imChange && imSlide && !logChange) {
			alertSlideValue = alertSlideValue + 1;
		}
	
		if (imChange && imPop) {
			alertPopValue = alertPopValue + 1;
		}
	
		if (imChange && imSound) {
			alertSound(1);
		}

		if (alertSlideValue) {
			try {
				alertSlide.doAlert((alertSlideValue & 1),(alertSlideValue & 2));
			}
			catch (e) { 
				logger(e, true); 
			}
		}
		
		if (alertPopValue) {
			alertHandler.alertPop((alertSlideValue & 1),(alertSlideValue & 2));
		}
	}
}

var popup = function(title, text) {  
	try {  
		Components.classes['@mozilla.org/alerts-service;1'].  
							getService(Components.interfaces.nsIAlertsService).  
							showAlertNotification(null, title, text, false, '', null);  
	} catch(e) {  
		// prevents runtime error on platforms that don't implement nsIAlertsService  
	}  
} 

var alertSlide = {
	alertServ:null,

	init: function() {
		// If already initialized, exit
		if (this.alertServ)
			return;
	
		// Check whether or not alerts are supported
		try {
			this.alertServ = Cc["@mozilla.org/alerts-service;1"].getService(Ci.nsIAlertsService);
		}

		catch(e) {
			this.alertServ = null;
			var imAlrChk = Prefs.imSlide;
			var topAlrChk = Prefs.topSlide;

			if (imAlrChk || topAlrChk) {
				Prefs.imSlide = false;
				Prefs.topSlide = false;
				popup("DSLR Notifier", "Your platform does not support alert sliders. They have been disabled.");
			}
		}
	},

	doAlert: function(im,news) {
		if (!this.alertServ) return;
	
		var repeatImSlide = Prefs.repeatImSlide;
		var repeatTopSlide = Prefs.repeatTopSlide;
	
		var im_text = dslrStatus.imText + " Instant Message" + ((dslrStatus.imText != "1") ? "s" : "");
		var news_text = dslrStatus.newsText + " Updated Topic" + ((dslrStatus.newsText != "1") ? "s" : "");
	
		if (im && news) {
			try {
				if ((repeatImSlide && repeatTopSlide) || (!dslrStatus.imslideonce && !dslrStatus.topslideonce)) {
					this.alertServ.showAlertNotification("chrome://dslrnotifier2/skin/images/imsg.gif", im_text + " & " + news_text, "Go To Inbox & Updated Topic", true, "", this, "DSLR Notifier");
					dslrStatus.imslideonce = true;
					dslrStatus.topslideonce = true;
					repeatImSlide = false;
					repeatTopSlide = false;
				}
			}
			catch(e) { logger(e, true); }
		}
	
		if (im) {
			try {
				if (repeatImSlide || !dslrStatus.imslideonce) {
					this.alertServ.showAlertNotification("chrome://dslrnotifier2/skin/images/imsg.gif", im_text, "Go To Inbox", true, "", this, "DSLR Notifier");
					dslrStatus.imslideonce = true;
				}
			}
			catch(e) { logger(e, true); }
		}
	
		if (news) {
			try {
				if (repeatTopSlide || !dslrStatus.topslideonce) {
					this.alertServ.showAlertNotification("chrome://dslrnotifier2/skin/images/camb.gif", news_text, "Click To View", true, "", this, "DSLR Notifier");
					dslrStatus.topslideonce = true;
				}
			}
			catch(e) {logger(e, true); }
		}
	},

	observe: function(aSubject, aTopic, aData)
	{
		if (aTopic == "alertclickcallback") 
		{
			if (dslrStatus.imFlag || dslrStatus.newsFlag) {
				// Find non-private window (unless in permanent private browsing) or open a new one
				var win = Privacy.getWindow(true);
		
				function doAction(window) {
					if ("gDslrNotifier" in window)  {
						if (dslrStatus.imFlag && dslrStatus.newsFlag)
							window.gDslrNotifier.dslrAct('tab','hybrid','kill',true);
						else if (dslrStatus.imFlag)
							window.gDslrNotifier.dslrAct('tab','im','kill',true);
						else
							window.gDslrNotifier.dslrAct('tab','news','kill',true);
					}
				}
				
				// If window not loaded, wait for it to load
				if (win.document.readyState == "complete")
					doAction(win);
				else {
					win.addEventListener("load", function() {
						win.removeEventListener("load", arguments.callee, false);
						// After the window loads selected or adding a tab usually fails so run asynchronously
						let runnable = {
							run: function()
							{
								doAction(win);
							}
						};
						Services.tm.currentThread.dispatch(runnable, Ci.nsIEventTarget.DISPATCH_NORMAL);
					}, false);
				}
			}
			alertHandler.closeAlert();
		}
	}
};

var alertHandler = {
	alertPop: function(im, news) {
		var openWindow = false;
		this.alertWindow = null;

		try {
			var params = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
			
			if (im && !dslrStatus.impoponce && news && !dslrStatus.toppoponce) {
				params.data = "hybrid";
				openWindow = true;
				dslrStatus.impoponce = true;
				dslrStatus.toppoponce = true;
			}
		
			else if (im && !dslrStatus.impoponce) {
				params.data = "im";
				openWindow = true;
				dslrStatus.impoponce = true;
			}
		
			else if (news && !dslrStatus.toppoponce) {
				params.data = "news";
				openWindow = true;
				dslrStatus.toppoponce = true;
			}

			if (openWindow) {
				this.alertWindow = Services.ww.openWindow(null,"chrome://dslrnotifier2/content/dslrpopup.xul","DSLRpop","centerscreen, chrome, alwaysRaised", params);
			}
		} catch (ex) { logger(ex, true); }
	},
	
	closeAlert: function() {
		if (this.alertWindow) {
			try {
				this.alertWindow.close();
			}
			catch(ex) {}
			this.alertWindow = null;
		}
	}
}

var alertSound = function(sound) {
	
	var prep;
	var repeat = false;
	
	switch (sound) {
		case 1:
			prep = Prefs.imUrl;
			repeat = Prefs.repeatImSound;
			break;
		
		case 2:
			prep = Prefs.topUrl;
			repeat = Prefs.repeatTopSound;
			break;
	}

	if (prep) {
		var soundURL = "file://"+prep;
	
		var ringOnce = (sound == 1) ? dslrStatus.imringOnce : dslrStatus.topringOnce;
		if (repeat || !ringOnce) {
			
			if (sound == 1)
				dslrStatus.imringOnce = true;
			else
				dslrStatus.topringOnce = true;

			try {
				let window = Privacy.getWindow(false, true);
				if (window) {
					var gSound = new window.Audio(soundURL);
					gSound.play();
				}
			} catch(ex) { logger(ex, true); }
		}
	}
};
